package com.iqball.app

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import com.google.gson.Gson
import com.iqball.app.api.EitherBodyConverter
import com.iqball.app.api.EitherCallAdapterFactory
import com.iqball.app.api.service.IQBallService
import com.iqball.app.page.HomePage
import com.iqball.app.page.LoginPage
import com.iqball.app.page.RegisterPage
import com.iqball.app.session.DataSession
import com.iqball.app.session.Session
import com.iqball.app.ui.theme.IQBallTheme
import okhttp3.OkHttpClient
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import retrofit2.create

class MainActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        val gson = Gson()

        val retrofit = Retrofit.Builder()
            .addConverterFactory(EitherBodyConverter.create())
            .addConverterFactory(GsonConverterFactory.create(gson))
            .addCallAdapterFactory(EitherCallAdapterFactory.create(gson))
            .baseUrl("https://iqball.maxou.dev/api/dotnet-master/")
            .client(
                OkHttpClient.Builder()
                    .addInterceptor { it.proceed(it.request()) }
                    .build()
            )
            .build()

        val service = retrofit.create<IQBallService>()

        setContent {
            IQBallTheme {
                // A surface container using the 'background' color from the theme
                Surface(
                    modifier = Modifier.fillMaxSize(),
                    color = MaterialTheme.colorScheme.background
                ) {
                    val sessionState = remember { mutableStateOf<Session>(DataSession()) }
                    App(service, sessionState)
                }
            }
        }
    }
}

@Composable
fun App(service: IQBallService, sessionState: MutableState<Session>) {

    val registerPage: @Composable () -> Unit = {
        RegisterPage(service = service, onRegisterSuccess = { auth ->
            sessionState.value = DataSession(auth)
        })
    }

    val loginPage: @Composable () -> Unit = {
        LoginPage(service = service, onLoginSuccess = { auth ->
            sessionState.value = DataSession(auth)
        })
    }
    val homePage : @Composable () -> Unit = { HomePage(service, sessionState.value) }

    //registerPage()
    val currentPage = remember(sessionState.value.auth) { if (sessionState.value.auth == null) loginPage else homePage }
    currentPage()

}