package com.iqball.app.api

import android.os.Build
import androidx.annotation.RequiresApi
import arrow.core.Either
import com.google.gson.Gson
import com.skydoves.retrofit.adapters.arrow.EitherCallAdapterFactory
import retrofit2.Call
import retrofit2.CallAdapter
import retrofit2.Retrofit
import java.lang.reflect.ParameterizedType
import java.lang.reflect.Type

class EitherCallAdapterFactory(private val gson: Gson) : CallAdapter.Factory() {
    override fun get(
        returnType: Type,
        annotations: Array<out Annotation>,
        retrofit: Retrofit
    ): CallAdapter<*, *>? {
        if (returnType !is ParameterizedType) {
            return null
        }
        if (returnType.rawType != Call::class.java) {
            return null
        }
        val eitherType = returnType.actualTypeArguments.first()
        if (eitherType !is ParameterizedType || eitherType.rawType != Either::class.java) {
            return null
        }
        return object : CallAdapter<Any, Any> {
            override fun responseType(): Type = returnType

            override fun adapt(call: Call<Any>): EitherCall<Any, Any> {
                return EitherCall(gson, eitherType, call)
            }
        }
    }

    companion object {
        fun create(gson: Gson) = EitherCallAdapterFactory(gson)
    }
}